@@ -14,6 +14,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import enum
 import logging
 import pipes
 import shlex
@@ -22,15 +23,15 @@ import subprocess
 import sys
 
 from gi.repository import Gdk
+from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import GObject
-from gi.repository import Gio
 from gi.repository import Gtk
 
-from . import task
-
 from meld.conf import _
+from meld.recent import RecentType
 from meld.settings import settings
+from meld.task import FifoScheduler
 
 log = logging.getLogger(__name__)
 
@@ -50,13 +51,32 @@ def make_custom_editor_command(path, line=0):
     return shlex.split(cmd)
 
 
-class MeldDoc(GObject.GObject):
+class ComparisonState(enum.IntEnum):
+    # TODO: Consider use-cases for states in gedit-enum-types.c
+    Normal = 0
+    Closing = 1
+    SavingError = 2
+
+
+class LabeledObjectMixin(GObject.GObject):
+    __gsignals__ = {
+        'label-changed': (
+            GObject.SignalFlags.RUN_FIRST, None,
+            (GObject.TYPE_STRING, GObject.TYPE_STRING)),
+    }
+
+    label_text = _("untitled")
+    tooltip_text = None
+
+    def label_changed(self):
+        self.emit("label-changed", self.label_text, self.tooltip_text)
+
+
+class MeldDoc(LabeledObjectMixin, GObject.GObject):
     """Base class for documents in the meld application.
     """
 
     __gsignals__ = {
-        'label-changed':        (GObject.SignalFlags.RUN_FIRST, None,
-                                 (GObject.TYPE_STRING, GObject.TYPE_STRING)),
         'file-changed':         (GObject.SignalFlags.RUN_FIRST, None,
                                  (GObject.TYPE_STRING,)),
         'create-diff':          (GObject.SignalFlags.RUN_FIRST, None,
@@ -69,18 +89,29 @@ class MeldDoc(GObject.GObject):
         'next-diff-changed':    (GObject.SignalFlags.RUN_FIRST, None,
                                  (bool, bool)),
         'close': (GObject.SignalFlags.RUN_FIRST, None, (bool,)),
+        'state-changed': (GObject.SignalFlags.RUN_FIRST, None, (int, int)),
     }
 
     def __init__(self):
-        GObject.GObject.__init__(self)
-        self.scheduler = task.FifoScheduler()
+        super().__init__()
+        self.scheduler = FifoScheduler()
         self.num_panes = 0
-        self.label_text = _("untitled")
-        self.tooltip_text = _("untitled")
         self.main_actiongroup = None
+        self._state = ComparisonState.Normal
 
-    def get_comparison(self):
-        """Get the comparison type and path(s) being compared"""
+    @property
+    def state(self):
+        return self._state
+
+    @state.setter
+    def state(self, value):
+        if value == self._state:
+            return
+        self.emit('state-changed', self._state, value)
+        self._state = value
+
+    def get_comparison(self) -> RecentType:
+        """Get the comparison type and URI(s) being compared"""
         pass
 
     def save(self):
@@ -167,9 +198,6 @@ class MeldDoc(GObject.GObject):
     def on_file_changed(self, filename):
         pass
 
-    def label_changed(self):
-        self.emit("label-changed", self.label_text, self.tooltip_text)
-
     def set_labels(self, lst):
         pass
 
@@ -195,7 +223,7 @@ class MeldDoc(GObject.GObject):
         self.popup_menu = None
         self.ui_merge_id = None
 
-    def on_delete_event(self, appquit=0):
+    def on_delete_event(self):
         """Called when the docs container is about to close.
 
         A doc normally returns Gtk.ResponseType.OK, but may instead return
